@@ -1,5 +1,7 @@
 package Module::Build;
 
+# $Id $
+
 # This module doesn't do much of anything itself, it inherits from the
 # modules that do the real work.  The only real thing it has to do is
 # figure out which OS-specific module to pull in.  Many of the
@@ -7,8 +9,12 @@ package Module::Build;
 # done in Module::Build::Base.
 
 use strict;
+use File::Spec ();
+use File::Path ();
+use File::Basename ();
+
 use vars qw($VERSION @ISA);
-$VERSION = '0.04';
+$VERSION = '0.05';
 
 # Okay, this is the brute-force method of finding out what kind of
 # platform we're on.  I don't know of a systematic way.  These values
@@ -53,8 +59,6 @@ my %OSTYPES = qw(
 		 mpeix     MPEiX
 		);
 
-use File::Spec;
-
 # We only use this once - don't waste a symbol table entry on it.
 # More importantly, don't make it an inheritable method.
 my $load = sub {
@@ -109,13 +113,17 @@ C<Module::Build> for its installation process, do the following:
    ./Build test
    ./Build install
 
-Other actions so far include:
+Actions defined so far include:
+
+  build                          help        
+  clean                          install     
+  dist                           manifest    
+  distcheck                      realclean   
+  distclean                      skipcheck   
+  distdir                        test        
+  disttest                       testdb      
+  fakeinstall                                
 
-   ./Build clean
-   ./Build realclean
-   ./Build fakeinstall
-   ./Build dist
-   ./Build help
 
 It's like the C<MakeMaker> metaphor, except that C<Build> is a short
 Perl script, not a long Makefile.  State is stored in a directory called
@@ -132,6 +140,9 @@ take some time, though.  In the meantime, I may implement some
 pass-through functionality so that unknown actions are passed to
 MakeMaker.
 
+For information on providing backward compatibility with
+C<ExtUtils::MakeMaker>, see L<Module::Build::Compat>.
+
 =head1 METHODS
 
 I list here some of the most important methods in the
@@ -143,29 +154,70 @@ this module in the first place was so that subclassing is possible
 (and easy), I will certainly write more docs as the interface
 stabilizes.
 
+=head2 $m = Module::Build->new(...)
+
+Creates a new Module::Build object.  Arguments to the new() method are
+listed below.  The only required argument is the C<module_name> argument.
+
 =over 4
 
-=item * $m = Module::Build->new(...)
+=item * module_name
 
-Creates a new Module::Build object.  The C<module_name> argument is
-required, and should be a string like C<'Your::Module'>.  The
-C<module_version> argument is optional - if not explicitly provided,
-we'll look for the version string in the module specified by
+The C<module_name> argument is required, and should be a string like
+C<'Your::Module'>.  We use it for several purposes, including finding
+the version string for this distribution, and creating a
+suitably-named distribution directory.
+
+=item * module_version
+
+The C<module_version> argument is optional - if not explicitly
+provided, we'll look for the version string in the module specified by
 C<module_name>, parsing it out according to the same rules as
 C<ExtUtils::MakeMaker> and C<CPAN.pm>.
 
+=item * module_version_from
+
+Allows you to specify an alternate file for finding the module
+version, instead of looking in the file specified by C<module_name>.
+
+=item * prereq
+
+An optional C<prereq> argument specifies any module prerequisites that
+the current module depends on.  The prerequisites are given in a hash
+reference, where the keys are the module names and the values are
+version specifiers:
+
+ prereq => {Foo::Module => '2.4',
+            Bar::Module => 0,
+            Ken::Module => '>= 1.2, != 1.5, < 2.0'},
+
+These three version specifiers have different effects.  The value
+C<'2.4'> means that B<at least> version 2.4 of C<Foo::Module> must be
+installed.  The value C<0> means that B<any> version of C<Bar::Module>
+is acceptable, even if C<Bar::Module> doesn't define a version.  The
+more verbose value C<'E<gt>= 1.2, != 1.5, E<lt> 2.0'> means that
+C<Ken::Module>'s version must be B<at least> 1.2, B<less than> 2.0,
+and B<not equal to> 1.5.  The list of criteria is separated by commas,
+and all criteria must be satisfied.
+
+=item * c_source
+
 An optional C<c_source> argument specifies a directory which contains
 C source files that the rest of the build may depend on.  Any C<.c>
 files in the directory will be compiled to object files.  The
 directory will be added to the search path during the compilation and
 linking phases of any C or XS files.
 
+=item * autosplit
+
 An optional C<autosplit> argument specifies a file which should be run
 through the C<Autosplit::autosplit()> function.  In general I don't
 consider this a great idea, and I may even go so far as to remove this
 feature later.  Let me know if I shouldn't.
 
-=item * $m->add_to_cleanup
+=back
+
+=head2 $m->add_to_cleanup
 
 A C<Module::Build> method may call C<< $self->add_to_cleanup(@files) >>
 to tell C<Module::Build> that certain files should be removed when the
@@ -175,7 +227,7 @@ static lists can get difficult to manage.  I preferred to keep the
 responsibility for registering temporary files close to the code that
 creates them.
 
-=item * Module::Build->resume
+=head2 Module::Build->resume
 
 You'll probably never call this method directly, it's only called from
 the auto-generated C<Build> script.  The C<new()> method is only
@@ -183,7 +235,7 @@ called once, when the user runs C<perl Build.PL>.  Thereafter, when
 the user runs C<Build test> or another action, the C<Module::Build>
 object is created using the C<resume()> method.
 
-=item * $m->dispatch
+=head2 $m->dispatch
 
 This method is also called from the auto-generated C<Build> script.
 It parses the command-line arguments into an action and an argument
@@ -193,7 +245,7 @@ C<ACTION_foo> method.  All arguments (including everything mentioned
 in L<ACTIONS> below) are contained in the C<< $self->{args} >> hash
 reference.
 
-=item * $m->os_type
+=head2 $m->os_type
 
 If you're subclassing Module::Build and some code needs to alter its
 behavior based on the current platform, you may only need to know
@@ -204,8 +256,33 @@ whatever is appropriate.  If you're running on an unknown platform, it
 will return C<undef> - there shouldn't be many unknown platforms
 though.
 
-=back
+=head2 $m->check_installed_version($module, $version)
 
+This method returns true or false, depending on whether (at least)
+version C<$version> of module C<$module> is installed.  The C<$module>
+argument is given as a string like C<"Data::Dumper">, and the
+C<$version> argument can take any of the forms described in L<prereq>
+above.  This allows very fine-grained version checking.
+
+If the check fails, we return false and set C<$@> to an informative
+error message.
+
+If the check succeeds, the return value is the actual version of
+C<$module> installed on the system.  This allows you to do the
+following:
+
+ my $installed = $m->check_installed_version('DBI', '1.15');
+ if ($installed) {
+   print "Congratulations, version $installed of DBI is installed.\n";
+ } else {
+   die "Sorry, you must install DBI.\n";
+ }
+
+If C<$version> is any nontrue value (notably zero) and any version of
+C<$module> is installed, we return true.  In this case, if C<$module>
+doesn't define a version, or if its version is zero, we return the
+special value "0 but true", which is numerically zero, but logically
+true.
 
 =head1 ACTIONS
 
@@ -230,6 +307,12 @@ The following build actions are provided by default.
 
 =over 4
 
+=item * help
+
+This action will simply print out a message that is meant to help you
+use the build process.  It will show you a list of available build
+actions too.
+
 =item * build
 
 This is analogous to the MakeMaker 'make' target with no arguments.
@@ -262,11 +345,19 @@ with C<.t> in a C<t/> directory.
 If you want tests to be 'verbose', i.e. show details of test execution
 rather than just summary information, pass the argument C<verbose=1>.
 
+If you want to run tests under the perl debugger, pass the argument
+C<debugger=1>.
+
 In addition, if a file called C<visual.pl> exists in the top-level
 directory, this file will be executed as a Perl script and its output
 will be shown to the user.  This is a good place to put speed tests or
 other tests that don't use the C<Test::Harness> format for output.
 
+=item * testdb
+
+This is a synonym for the 'test' action with the C<debugger=1>
+argument.
+
 =item * clean
 
 This action will clean up any files that the build process may have
@@ -304,34 +395,65 @@ This is just like the C<install> action, but it won't actually do
 anything, it will just report what it I<would> have done if you had
 actually run the C<install> action.
 
+=item * manifest
+
+This is an action intended for use by module authors, not people
+installing modules.  It will bring the F<MANIFEST> up to date with the
+files currently present in the distribution.  You may use a
+F<MANIFEST.SKIP> file to exclude certain files or directories from
+inclusion in the F<MANIFEST>.  F<MANIFEST.SKIP> should contain a bunch
+of regular expressions, one per line.  If a file in the distribution
+directory matches any of the regular expressions, it won't be included
+in the F<MANIFEST>.
+
+The following is a reasonable F<MANIFEST.SKIP> starting point, you can
+add your own stuff to it:
+
+   ^_build
+   ^Build$
+   ^blib
+   ~$
+   \.bak$
+   ^MANIFEST\.SKIP$
+   CVS
+
+See the L<distcheck> and L<skipcheck> actions if you want to find out
+what the C<manifest> action would do, without actually doing anything.
+
 =item * dist
 
 This action is helpful for module authors who want to package up their
 module for distribution through a medium like CPAN.  It will create a
-tarball of the files listed in MANIFEST and compress the tarball using
+tarball of the files listed in F<MANIFEST> and compress the tarball using
 GZIP compression.
 
-=item * manifest
+=item * distcheck
 
-This is an action intended for use by module authors, not people
-installing modules.  It will bring the MANIFEST up to date with the
-files currently present in the distribution.  You may use a
-MANIFEST.SKIP file to exclude certain files or directories from
-inclusion in the MANIFEST.  MANIFEST.SKIP should contain a bunch of
-regular expressions, one per line.  If a file in the distribution
-directory matches any of the regular expressions, it won't be included
-in the MANIFEST.
+Reports which files are in the build directory but not in the
+F<MANIFEST> file, and vice versa. (See L<manifest> for details)
 
-(Note to self: it would be nice to have a 'fake_manifest' action that
-would just go through the motions of adding to MANIFEST but not
-actually do anything.  Currently ExtUtils::Manifest doesn't support
-it, though.)
+=item * skipcheck
 
-=item * help
+Reports which files are skipped due to the entries in the
+F<MANIFEST.SKIP> file (See L<manifest> for details)
 
-This action will simply print out a message that is meant to help you
-use the build process.  It will show you a list of available build
-actions too.
+=item * distclean
+
+Performs the 'realclean' action and then the 'distcheck' action.
+
+=item * distdir
+
+Creates a directory called C<$(DISTNAME)-$(VERSION)> (if that
+directory already exists, it will be removed first).  Then copies all
+the files listed in the F<MANIFEST> file to that directory.  This
+directory is what people will see when they download your distribution
+and unpack it.
+
+=item * disttest
+
+Performs the 'distdir' action, then switches into that directory and
+runs a C<perl Build.PL>, followed by the 'build' and 'test' actions in
+that directory.
 
 =back
 
@@ -384,11 +506,52 @@ Here is the parent-child class hierarchy in classy ASCII art:
    |   Module::Build::Base    |  (Most of the functionality of 
    \--------------------------/   Module::Build is defined here.)
 
+=head1 SUBCLASSING
 
-Right now, if you want to subclass Module::Build you must do so by
-including an actual .pm file somewhere in your distribution.  There
-will be much better ways to do this in the future.  Can't do
-everything at once...
+Right now, there are two ways to subclass Module::Build.  The first
+way is to create a regular module (in a C<.pm> file) that inherits
+from Module::Build, and use that module's class instead of using
+Module::Build directly:
+
+  ------ in Build.PL: ----------
+  #!/usr/bin/perl
+  
+  use lib qw(/nonstandard/library/path);
+  use My::Builder;  # Or whatever you want to call it
+  
+  my $m = My::Builder->new(module_name => 'Next::Big::Thing');
+  $m->create_build_script;
+
+This is relatively straightforward, and is the best way to do things
+if your My::Builder class contains lots of code.  The
+C<create_build_script()> method will ensure that the current value of
+C<@INC> (including the C</nonstandard/library/path>) is propogated to
+the Build script, so that My::Builder can be found when running build
+actions.
+
+For very small additions, Module::Build provides a C<subclass()>
+method that lets you subclass Module::Build more conveniently, without
+creating a separate file for your module:
+
+  ------ in Build.PL: ----------
+  #!/usr/bin/perl
+  
+  my $class = Module::Build->subclass
+    (
+     class => 'My::Builder',
+     code => q{
+      sub ACTION_foo {
+        print "I'm fooing to death!\n";
+      }
+     },
+    );
+  
+  my $m = $class->new(module_name => 'Module::Build');
+  $m->create_build_script;
+
+Behind the scenes, this actually does create a C<.pm> file, since the
+code you provide must persist after Build.PL is run if it is to be
+very useful.
 
 
 =head1 MOTIVATIONS
@@ -448,7 +611,7 @@ worrying about backward compatibility.
 Finally, Perl is said to be a language for system administration.
 Could it really be the case that Perl isn't up to the task of building
 and installing software?  Absolutely not - see the C<Cons> package for
-one example, at L<http://www.dsmit.com/cons/> .
+one example, at L<"http://www.dsmit.com/cons/"> .
 
 =back
 
@@ -456,18 +619,6 @@ Please contact me if you have any questions or ideas.
 
 =head1 TO DO
 
-Need to implement a prerequisite mechanism, similar to MakeMaker's
-C<PREREQ_PM> stuff.
-
-There will also be a subclassing mechanism that doesn't require as
-much module infrastructure to use.  Something like this:
-
- use Module::Build subclass => <<'EOF';
-  sub ACTION_foo {
-    ... implement the 'foo' action ...
-  }
- EOF
-
 The current method of relying on time stamps to determine whether a
 derived file is out of date isn't likely to scale well, since it
 requires tracing all dependencies backward, it runs into problems on
@@ -479,9 +630,13 @@ can make 'widowed' files by doing C<Build>, C<perl Build.PL>, and then
 C<Build realclean>.  Should be easy to fix, but it's got me wondering
 whether the dynamic declaration of dependencies is a good idea.
 
+- make man pages and install them.
+- append to perllocal.pod
+- write .packlist in appropriate location (needed for un-install)
+
 =head1 AUTHOR
 
-Ken Williams, ken@forum.swarthmore.edu
+Ken Williams, ken@mathforum.org
 
 =head1 SEE ALSO
 
