@@ -1,5 +1,7 @@
 package Module::Build::Base;
 
+# $Id $
+
 use strict;
 use Config;
 use File::Copy ();
@@ -24,10 +26,11 @@ sub new {
 
   $self->{args} = {%Config, @_, %$args};
 
-  $self->find_version;
-  $self->write_config;
-  
   $self->check_manifest;
+  $self->check_prereq;
+  $self->find_version;
+  
+  $self->write_config;
   
   return $self;
 }
@@ -41,6 +44,44 @@ sub resume {
   return $self;
 }
 
+# XXX Problem - if Module::Build is loaded from a different directory,
+# it'll look for (and perhaps destroy/create) a _build directory.
+sub subclass {
+  my ($pack, %opts) = @_;
+
+  my $build_dir = '_build'; # XXX The _build directory is ostensibly settable by the user.  Shouldn't hard-code here.
+  $pack->delete_filetree($build_dir) if -e $build_dir;
+
+  die "Must provide 'code' or 'class' option to subclass()\n"
+    unless $opts{code} or $opts{class};
+
+  $opts{code}  ||= '';
+  $opts{class} ||= 'MyModuleBuilder';
+  
+  my $filename = File::Spec->catfile($build_dir, 'lib', split '::', $opts{class}) . '.pm';
+  my $filedir  = File::Basename::dirname($filename);
+  print "Creating custom builder $filename in $filedir\n";
+  
+  File::Path::mkpath($filedir);
+  die "Can't create directory $filedir: $!" unless -d $filedir;
+  
+  open my($fh), ">$filename" or die "Can't create $filename: $!";
+  print $fh <<EOF;
+package $opts{class};
+use Module::Build;
+\@ISA = qw(Module::Build);
+$opts{code}
+1;
+EOF
+  close $fh;
+  
+  push @INC, File::Spec->catdir($build_dir, 'lib');
+  eval "use $opts{class}";
+  die $@ if $@;
+
+  return $opts{class};
+}
+
 sub find_version {
   my ($self) = @_;
   return if exists $self->{args}{module_version};
@@ -85,12 +126,11 @@ sub version_from_file {
 		    }; \$$2
 		   };
       local $^W;
-      my $result = eval $eval;
-      die "Could not eval '$_' in '$file': $@" if $@;
-      return $result;
+      return scalar eval $eval;
     }
   }
-  die "Couldn't find version string in '$file'";
+  return undef;
+  #die "Couldn't find version string in '$file'";
 }
 
 sub add_to_cleanup {
@@ -144,9 +184,8 @@ sub read_config {
 sub write_config {
   my ($self) = @_;
   
-  $self->delete_filetree($self->{config_dir});
-  mkdir $self->{config_dir}, 0777
-    or die "Can't mkdir $self->{config_dir}: $!";
+  File::Path::mkpath($self->{config_dir});
+  -d $self->{config_dir} or die "Can't mkdir $self->{config_dir}: $!";
   
   my $file = $self->config_file('build_params');
   open my $fh, ">$file" or die "Can't create '$file': $!";
@@ -164,6 +203,60 @@ sub write_config {
   }
 }
 
+sub check_prereq {
+  my $self = shift;
+  return 1 unless $self->{prereq};
+
+  my $pass = 1;
+  while (my ($modname, $spec) = each %{$self->{prereq}}) {
+    my $thispass = $self->check_installed_version($modname, $spec);
+    warn "WARNING: $@\n" unless $thispass;
+    $pass &&= $thispass;
+  }
+
+  if (!$pass) {
+    warn "ERRORS FOUND IN PREREQUISITES.  You may wish to install the versions ".
+         "of the modules indicated above before proceeding with this installation.\n";
+  }
+  return $pass;
+}
+
+sub check_installed_version {
+  my ($self, $modname, $spec) = @_;
+
+  my $file = $self->module_name_to_file($modname);
+  unless ($file) {
+    $@ = "Prerequisite $modname isn't installed";
+    return 0;
+  }
+
+  my $version = $self->version_from_file($file);
+  if ($spec and !$version) {
+    $@ = "Couldn't find a \$VERSION in prerequisite '$file'";
+    return 0;
+  }
+
+  my @conditions;
+  if ($spec =~ /^\s*([\w.]+)\s*$/) { # A plain number, maybe with dots, letters, and underscores
+    @conditions = (">= $spec");
+  } else {
+    @conditions = split /\s*,\s*/, $self->{prereq}{$modname};
+  }
+
+  foreach (@conditions) {
+    if ($_ !~ /^\s*  (<=?|>=?|==|!=)  \s*  [\w.]+  \s*$/x) {
+      $@ = "Invalid prerequisite condition for $modname: $_";
+      next;
+    }
+    unless (eval "\$version $_") {
+      $@ = "$modname version $version is installed, but we need version $_";
+      return 0;
+    }
+  }
+
+  return $version ? $version : '0 but true';
+}
+
 sub rm_previous_build_script {
   my $self = shift;
   if (-e $self->{build_script}) {
@@ -179,25 +272,34 @@ sub make_build_script_executable {
 sub print_build_script {
   my ($self, $fh) = @_;
   
-  my $quoted_INC = join ', ', map "'$_'", @INC;
-  my $build_dir = File::Spec->rel2abs(File::Basename::dirname($0));
   my $build_package = ref($self);
 
+  my ($config_dir, $build_script, $build_dir) = 
+    ($self->{config_dir}, $self->{build_script},
+     File::Spec->rel2abs(File::Basename::dirname($0)));  # XXX should be property of $self
+
+  my @myINC = @INC;
+  for ($config_dir, $build_script, $build_dir, @myINC) {
+    s/([\\\'])/\\$1/g;
+  }
+
+  my $quoted_INC = join ', ', map "'$_'", @myINC;
+
   print $fh <<EOF;
 $self->{args}{startperl} -w
 
 BEGIN { \@INC = ($quoted_INC) }
 
-chdir('$build_dir');
+chdir('$build_dir') or die 'Cannot chdir to $build_dir: '.\$!;
 use $build_package;
 
 my \$build = resume $build_package (
-  config_dir => '$self->{config_dir}',
-  build_script => '$self->{build_script}',
+  config_dir => '$config_dir',
+  build_script => '$build_script',
 );
 eval {\$build->dispatch};
 my \$err = \$@;
-\$build->write_cleanup;
+\$build->write_cleanup;  # Always write, even if error occurs
 die \$err if \$err;
 
 EOF
@@ -224,8 +326,8 @@ sub check_manifest {
   # could easily be re-written into a modern Perl dialect.
 
   print "Checking whether your kit is complete...\n";
-  require ExtUtils::Manifest;
-  $ExtUtils::Manifest::Quiet = 1;
+  require ExtUtils::Manifest;  # ExtUtils::Manifest is not warnings clean.
+  local ($^W, $ExtUtils::Manifest::Quiet) = (0,1);
   
   if (my @missed = ExtUtils::Manifest::manicheck()) {
     print "Warning: the following files are missing in your kit:\n";
@@ -271,23 +373,48 @@ sub cull_args {
   return ($action, \%args);
 }
 
+sub super_classes {
+  my ($self, $class, $seen) = @_;
+  $class ||= ref($self);
+  $seen  ||= {};
+  
+  no strict 'refs';
+  my @super = grep {not $seen->{$_}++} $class, @{ $class . '::ISA' };
+  return @super, map {$self->super_classes($_,$seen)} @super;
+}
+
 sub ACTION_help {
   my ($self) = @_;
-  # XXX the list of actions should be autogenerated somehow.
-  
+
   print <<EOF;
 
  Usage: $0 <action> arg1=value arg2=value ...
  Example: $0 test verbose=1
  
  Actions defined:
-  build                      clean
-  test                       realclean
-  dist                       install
-  help                       fakeinstall
-  manifest
-
 EOF
+
+  my %actions;
+  {
+    no strict 'refs';
+    
+    foreach my $class ($self->super_classes) {
+      #print "Checking $class\n";
+      foreach ( keys %{ $class . '::' } ) {
+	$actions{$1}++ if /ACTION_(\w+)/;
+      }
+    }
+  }
+
+  my @actions = sort keys %actions;
+  # Flow down columns, not across rows
+  @actions = map $actions[($_ + ($_ % 2) * @actions) / 2],  0..$#actions;
+  
+  while (my ($one, $two) = splice @actions, 0, 2) {
+    printf("  %-12s                   %-12s\n", $one, $two||'');
+  }
+
+  print "\nSee `perldoc Module::Build` for details of the individual actions.\n";
 }
 
 sub ACTION_test {
@@ -296,7 +423,8 @@ sub ACTION_test {
   
   $self->depends_on('build');
   
-  $Test::Harness::verbose = $self->{args}{verbose} || 0;
+  local $Test::Harness::switches = '-w -d' if $self->{args}{debugger};
+  local $Test::Harness::verbose = $self->{args}{verbose} || 0;
   local $ENV{TEST_VERBOSE} = $self->{args}{verbose} || 0;
 
   # Make sure we test the module in blib/
@@ -323,6 +451,12 @@ sub ACTION_test {
   }
 }
 
+sub ACTION_testdb {
+  my ($self) = @_;
+  local $self->{args}{debugger} = 1;
+  $self->depends_on('test');
+}
+
 sub ACTION_build {
   my ($self) = @_;
   
@@ -347,14 +481,14 @@ sub ACTION_build {
 
 sub ACTION_install {
   my ($self) = @_;
-  require ExtUtils::Install;  # Grr, uses MakeMaker
+  require ExtUtils::Install;
   $self->depends_on('build');
   ExtUtils::Install::install($self->install_map('blib'), 1, 0);
 }
 
 sub ACTION_fakeinstall {
   my ($self) = @_;
-  require ExtUtils::Install;  # Grr, uses MakeMaker
+  require ExtUtils::Install;
   $self->depends_on('build');
   ExtUtils::Install::install($self->install_map('blib'), 1, 1);
 }
@@ -374,25 +508,79 @@ sub ACTION_realclean {
 
 sub ACTION_dist {
   my ($self) = @_;
-  (my $dist_dir = $self->{args}{module_name}) =~ s/::/-/;
-  $dist_dir .= "-$self->{args}{module_version}";
-  mkdir $dist_dir or die "Can't create '$dist_dir/': $!";
   
-  require ExtUtils::Manifest;
-  my $dist_files = ExtUtils::Manifest::maniread('MANIFEST');
-  ExtUtils::Manifest::manicopy($dist_files, $dist_dir, 'best');
+  $self->depends_on('distdir');
+  
+  my $dist_dir = $self->dist_dir;
   
   $self->make_tarball($dist_dir);
   $self->delete_filetree($dist_dir);
 }
 
-sub ACTION_manifest {
+sub ACTION_distcheck {
   my ($self) = @_;
   
   require ExtUtils::Manifest;
+  local $^W; # ExtUtils::Manifest is not warnings clean.
+  ExtUtils::Manifest::fullcheck();
+}
+
+sub ACTION_skipcheck {
+  my ($self) = @_;
+  
+  require ExtUtils::Manifest;
+  local $^W; # ExtUtils::Manifest is not warnings clean.
+  ExtUtils::Manifest::skipcheck();
+}
+
+sub ACTION_distclean {
+  my ($self) = @_;
+  
+  $self->depends('realclean');
+  $self->depends('distcheck');
+}
+
+sub ACTION_distdir {
+  my ($self) = @_;
+  
+  my $dist_dir = $self->dist_dir;
+  
+  require ExtUtils::Manifest;  # ExtUtils::Manifest is not warnings clean.
+  local ($^W, $ExtUtils::Manifest::Quiet) = (0,1);
+  
+  my $dist_files = ExtUtils::Manifest::maniread('MANIFEST');
+  ExtUtils::Manifest::manicopy($dist_files, $dist_dir, 'best');
+}
+
+sub ACTION_disttest {
+  my ($self) = @_;
+
+  $self->depends_on('distdir');
+
+  my $dist_dir = $self->dist_dir;
+  chdir $dist_dir or die "Cannot chdir to $dist_dir: $!";
+  # XXX could be different names for scripts
+  $self->do_system("$^X Build.PL") or die "Error executing '$^X Build.PL' in dist directory: $!";
+  $self->do_system('./Build') or die "Error executing './Build' in dist directory: $!";
+  $self->do_system('./Build test') or die "Error executing './Build test' in dist directory: $!";
+  # XXX doesn't change back to top dir
+}
+
+sub ACTION_manifest {
+  my ($self) = @_;
+  
+  require ExtUtils::Manifest;  # ExtUtils::Manifest is not warnings clean.
+  local ($^W, $ExtUtils::Manifest::Quiet) = (0,1);
   ExtUtils::Manifest::mkmanifest();
 }
 
+sub dist_dir {
+  my ($self) = @_;
+
+  (my $dist_dir = $self->{args}{module_name}) =~ s/::/-/;
+  return "$dist_dir-$self->{args}{module_version}";
+}
+
 sub make_tarball {
   my ($self, $dir) = @_;
   
@@ -434,7 +622,7 @@ sub delete_filetree {
     next unless -e $_;
     print "Deleting $_\n";
     if (-d $_) {
-      File::Path::rmtree($_, 0, 1);
+      File::Path::rmtree($_, 0, 0);
     } else {
       unlink $_;
     }
